@@ -31,9 +31,9 @@
 import os
 import re
 import unittest
-from binascii import hexlify
+from binascii import hexlify, unhexlify
 
-from Cryptodome.Util.py3compat import b, tobytes, bchr, unhexlify
+from Cryptodome.Util.py3compat import b, tobytes, bchr, _memoryview
 from Cryptodome.Util.strxor import strxor_c
 from Cryptodome.Util.number import long_to_bytes
 from Cryptodome.SelfTest.st_common import list_test_cases
@@ -81,7 +81,7 @@ class OcbTests(unittest.TestCase):
                           nonce=b(""))
 
         # nonce can be up to 15 bytes long
-        for length in xrange(1, 16):
+        for length in range(1, 16):
             AES.new(self.key_128, AES.MODE_OCB, nonce=self.data_128[:length])
 
         self.assertRaises(ValueError, AES.new, self.key_128, AES.MODE_OCB,
@@ -148,7 +148,7 @@ class OcbTests(unittest.TestCase):
                           nonce=self.nonce_96, mac_len=16+1)
 
         # Valid MAC length
-        for mac_len in xrange(8, 16 + 1):
+        for mac_len in range(8, 16 + 1):
             cipher = AES.new(self.key_128, AES.MODE_OCB, nonce=self.nonce_96,
                              mac_len=mac_len)
             _, mac = cipher.encrypt_and_digest(self.data_128)
@@ -222,40 +222,102 @@ class OcbTests(unittest.TestCase):
             self.assertEquals(cipher.digest(), ref_mac)
 
     def test_bytearray(self):
+
         # Encrypt
+        key_ba = bytearray(self.key_128)
+        nonce_ba = bytearray(self.nonce_96)
+        header_ba = bytearray(self.data_128)
+        data_ba = bytearray(self.data_128)
+
         cipher1 = AES.new(self.key_128,
                           AES.MODE_OCB,
                           nonce=self.nonce_96)
         cipher1.update(self.data_128)
-        ref1 = cipher1.encrypt(self.data_128) + cipher1.encrypt()
-
-        cipher2 = AES.new(bytearray(self.key_128),
-                          AES.MODE_OCB,
-                          nonce=bytearray(self.nonce_96))
-        cipher2.update(bytearray(self.data_128))
-        ref2 = cipher2.encrypt(bytearray(self.data_128)) + cipher2.encrypt()
-
-        self.assertEqual(ref1, ref2)
-        self.assertEqual(cipher1.nonce, cipher2.nonce)
-
+        ct = cipher1.encrypt(self.data_128) + cipher1.encrypt()
         tag = cipher1.digest()
 
+        cipher2 = AES.new(key_ba,
+                          AES.MODE_OCB,
+                          nonce=nonce_ba)
+        key_ba[:3] = b"\xFF\xFF\xFF"
+        nonce_ba[:3] = b"\xFF\xFF\xFF"
+        cipher2.update(header_ba)
+        header_ba[:3] = b"\xFF\xFF\xFF"
+        ct_test = cipher2.encrypt(data_ba) + cipher2.encrypt()
+        data_ba[:3] = b"\xFF\xFF\xFF"
+        tag_test = cipher2.digest()
+
+        self.assertEqual(ct, ct_test)
+        self.assertEqual(tag, tag_test)
+        self.assertEqual(cipher1.nonce, cipher2.nonce)
+
         # Decrypt
-        cipher3 = AES.new(self.key_128,
+        key_ba = bytearray(self.key_128)
+        nonce_ba = bytearray(self.nonce_96)
+        header_ba = bytearray(self.data_128)
+        del data_ba
+
+        cipher4 = AES.new(key_ba,
+                          AES.MODE_OCB,
+                          nonce=nonce_ba)
+        key_ba[:3] = b"\xFF\xFF\xFF"
+        nonce_ba[:3] = b"\xFF\xFF\xFF"
+        cipher4.update(header_ba)
+        header_ba[:3] = b"\xFF\xFF\xFF"
+        pt_test = cipher4.decrypt_and_verify(bytearray(ct_test), bytearray(tag_test))
+
+        self.assertEqual(self.data_128, pt_test)
+
+    def test_memoryview(self):
+
+        # Encrypt
+        key_mv = memoryview(bytearray(self.key_128))
+        nonce_mv = memoryview(bytearray(self.nonce_96))
+        header_mv = memoryview(bytearray(self.data_128))
+        data_mv = memoryview(bytearray(self.data_128))
+
+        cipher1 = AES.new(self.key_128,
                           AES.MODE_OCB,
                           nonce=self.nonce_96)
-        cipher3.update(self.data_128)
-        ref3 = cipher3.decrypt(ref1) + cipher3.decrypt()
+        cipher1.update(self.data_128)
+        ct = cipher1.encrypt(self.data_128) + cipher1.encrypt()
+        tag = cipher1.digest()
 
-        cipher4 = AES.new(bytearray(self.key_128),
+        cipher2 = AES.new(key_mv,
                           AES.MODE_OCB,
-                          nonce=bytearray(self.nonce_96))
-        cipher4.update(bytearray(self.data_128))
-        ref4 = cipher4.decrypt(bytearray(ref1)) + cipher4.decrypt()
+                          nonce=nonce_mv)
+        key_mv[:3] = b"\xFF\xFF\xFF"
+        nonce_mv[:3] = b"\xFF\xFF\xFF"
+        cipher2.update(header_mv)
+        header_mv[:3] = b"\xFF\xFF\xFF"
+        ct_test = cipher2.encrypt(data_mv) + cipher2.encrypt()
+        data_mv[:3] = b"\xFF\xFF\xFF"
+        tag_test = cipher2.digest()
 
-        self.assertEqual(ref3, ref4)
+        self.assertEqual(ct, ct_test)
+        self.assertEqual(tag, tag_test)
+        self.assertEqual(cipher1.nonce, cipher2.nonce)
 
-        cipher3.verify(bytearray(tag))
+        # Decrypt
+        key_mv = memoryview(bytearray(self.key_128))
+        nonce_mv = memoryview(bytearray(self.nonce_96))
+        header_mv = memoryview(bytearray(self.data_128))
+        del data_mv
+
+        cipher4 = AES.new(key_mv,
+                          AES.MODE_OCB,
+                          nonce=nonce_mv)
+        key_mv[:3] = b"\xFF\xFF\xFF"
+        nonce_mv[:3] = b"\xFF\xFF\xFF"
+        cipher4.update(header_mv)
+        header_mv[:3] = b"\xFF\xFF\xFF"
+        pt_test = cipher4.decrypt_and_verify(memoryview(ct_test), memoryview(tag_test))
+
+        self.assertEqual(self.data_128, pt_test)
+
+    import sys
+    if sys.version[:3] == "2.6":
+        del test_memoryview
 
 
 class OcbFSMTests(unittest.TestCase):
@@ -379,13 +441,13 @@ class OcbFSMTests(unittest.TestCase):
         cipher = AES.new(self.key_128, AES.MODE_OCB, nonce=self.nonce_96)
         cipher.update(self.data_128)
         first_mac = cipher.digest()
-        for x in xrange(4):
+        for x in range(4):
             self.assertEqual(first_mac, cipher.digest())
 
         # Multiple calls to verify
         cipher = AES.new(self.key_128, AES.MODE_OCB, nonce=self.nonce_96)
         cipher.update(self.data_128)
-        for x in xrange(5):
+        for x in range(5):
             cipher.verify(first_mac)
 
     def test_valid_encrypt_and_digest_decrypt_and_verify(self):
@@ -646,7 +708,7 @@ class OcbRfc7253Test(unittest.TestCase):
             key = bchr(0) * (keylen // 8 - 1) + bchr(taglen)
             C = b("")
 
-            for i in xrange(128):
+            for i in range(128):
                 S = bchr(0) * i
 
                 N = long_to_bytes(3 * i + 1, 12)
